@@ -37,7 +37,7 @@
 #include "keyserver-internal.h"
 #include "call-agent.h"
 #include "host2net.h"
-
+#include "mbox-util.h"
 
 #define MAX_PK_CACHE_ENTRIES   PK_UID_CACHE_SIZE
 #define MAX_UID_CACHE_ENTRIES  PK_UID_CACHE_SIZE
@@ -116,7 +116,7 @@ print_stats ()
   for (i = 0; i < DIM (lkup_stats); i++)
     {
       if (lkup_stats[i].any)
-	fprintf (stderr,
+	es_fprintf (es_stderr,
 		 "lookup stats: mode=%-2d  ok=%-6d  nokey=%-6d  err=%-6d\n",
 		 i,
 		 lkup_stats[i].okay_count,
@@ -968,17 +968,26 @@ get_pubkey_byfpr (PKT_public_key *pk, const byte *fpr)
 }
 
 
-/* Search for a key with the given fingerprint.
+/* Search for a key with the given fingerprint.  The caller need to
+ * prove an allocated public key object at PK.  If R_KEYBLOCK is not
+ * NULL the entire keyblock is stored there and the caller needs to
+ * call release_kbnode() on it.  Note that this function does an exact
+ * search and thus the public key stored at PK may be a copy of a
+ * subkey.
+ *
  * FIXME:
  * We should replace this with the _byname function.  This can be done
  * by creating a userID conforming to the unified fingerprint style.
  */
 int
-get_pubkey_byfprint (PKT_public_key * pk,
+get_pubkey_byfprint (PKT_public_key *pk, kbnode_t *r_keyblock,
 		     const byte * fprint, size_t fprint_len)
 {
   int rc;
 
+  if (r_keyblock)
+    *r_keyblock = NULL;
+
   if (fprint_len == 20 || fprint_len == 16)
     {
       struct getkey_ctx_s ctx;
@@ -994,7 +1003,14 @@ get_pubkey_byfprint (PKT_public_key * pk,
       memcpy (ctx.items[0].u.fpr, fprint, fprint_len);
       rc = lookup (&ctx, &kb, 0);
       if (!rc && pk)
-	pk_from_block (&ctx, pk, kb);
+        {
+          pk_from_block (&ctx, pk, kb);
+          if (r_keyblock)
+            {
+              *r_keyblock = kb;
+              kb = NULL;
+            }
+        }
       release_kbnode (kb);
       get_pubkey_end (&ctx);
     }
@@ -2373,7 +2389,7 @@ finish_lookup (GETKEY_CTX ctx)
 	}
     }
 
-  if (DBG_CACHE)
+  if (DBG_LOOKUP)
     log_debug ("finish_lookup: checking key %08lX (%s)(req_usage=%x)\n",
 	       (ulong) keyid_from_pk (keyblock->pkt->pkt.public_key, NULL),
 	       foundk ? "one" : "all", req_usage);
@@ -2400,43 +2416,43 @@ finish_lookup (GETKEY_CTX ctx)
 	  if (foundk)
 	    nextk = NULL; /* what a hack */
 	  pk = k->pkt->pkt.public_key;
-	  if (DBG_CACHE)
+	  if (DBG_LOOKUP)
 	    log_debug ("\tchecking subkey %08lX\n",
 		       (ulong) keyid_from_pk (pk, NULL));
 	  if (!pk->flags.valid)
 	    {
-	      if (DBG_CACHE)
+	      if (DBG_LOOKUP)
 		log_debug ("\tsubkey not valid\n");
 	      continue;
 	    }
 	  if (pk->flags.revoked)
 	    {
-	      if (DBG_CACHE)
+	      if (DBG_LOOKUP)
 		log_debug ("\tsubkey has been revoked\n");
 	      continue;
 	    }
 	  if (pk->has_expired)
 	    {
-	      if (DBG_CACHE)
+	      if (DBG_LOOKUP)
 		log_debug ("\tsubkey has expired\n");
 	      continue;
 	    }
 	  if (pk->timestamp > curtime && !opt.ignore_valid_from)
 	    {
-	      if (DBG_CACHE)
+	      if (DBG_LOOKUP)
 		log_debug ("\tsubkey not yet valid\n");
 	      continue;
 	    }
 
 	  if (!((pk->pubkey_usage & USAGE_MASK) & req_usage))
 	    {
-	      if (DBG_CACHE)
+	      if (DBG_LOOKUP)
 		log_debug ("\tusage does not match: want=%x have=%x\n",
 			   req_usage, pk->pubkey_usage);
 	      continue;
 	    }
 
-	  if (DBG_CACHE)
+	  if (DBG_LOOKUP)
 	    log_debug ("\tsubkey might be fine\n");
 	  /* In case a key has a timestamp of 0 set, we make sure
 	     that it is used.  A better change would be to compare
@@ -2455,33 +2471,33 @@ finish_lookup (GETKEY_CTX ctx)
   if ((!latest_key && !(ctx->exact && foundk != keyblock)) || req_prim)
     {
       PKT_public_key *pk;
-      if (DBG_CACHE && !foundk && !req_prim)
+      if (DBG_LOOKUP && !foundk && !req_prim)
 	log_debug ("\tno suitable subkeys found - trying primary\n");
       pk = keyblock->pkt->pkt.public_key;
       if (!pk->flags.valid)
 	{
-	  if (DBG_CACHE)
+	  if (DBG_LOOKUP)
 	    log_debug ("\tprimary key not valid\n");
 	}
       else if (pk->flags.revoked)
 	{
-	  if (DBG_CACHE)
+	  if (DBG_LOOKUP)
 	    log_debug ("\tprimary key has been revoked\n");
 	}
       else if (pk->has_expired)
 	{
-	  if (DBG_CACHE)
+	  if (DBG_LOOKUP)
 	    log_debug ("\tprimary key has expired\n");
 	}
       else if (!((pk->pubkey_usage & USAGE_MASK) & req_usage))
 	{
-	  if (DBG_CACHE)
+	  if (DBG_LOOKUP)
 	    log_debug ("\tprimary key usage does not match: "
 		       "want=%x have=%x\n", req_usage, pk->pubkey_usage);
 	}
       else /* Okay.  */
 	{
-	  if (DBG_CACHE)
+	  if (DBG_LOOKUP)
 	    log_debug ("\tprimary key may be used\n");
 	  latest_key = keyblock;
 	  latest_date = pk->timestamp;
@@ -2490,13 +2506,13 @@ finish_lookup (GETKEY_CTX ctx)
 
   if (!latest_key)
     {
-      if (DBG_CACHE)
+      if (DBG_LOOKUP)
 	log_debug ("\tno suitable key found -  giving up\n");
       return 0; /* Not found.  */
     }
 
 found:
-  if (DBG_CACHE)
+  if (DBG_LOOKUP)
     log_debug ("\tusing key %08lX\n",
 	       (ulong) keyid_from_pk (latest_key->pkt->pkt.public_key, NULL));
 
@@ -2525,6 +2541,29 @@ found:
 }
 
 
+/* Return true if all the search modes are fingerprints.  */
+static int
+search_modes_are_fingerprint (getkey_ctx_t ctx)
+{
+  size_t n, found;
+
+  for (n=found=0; n < ctx->nitems; n++)
+    {
+      switch (ctx->items[n].mode)
+        {
+        case KEYDB_SEARCH_MODE_FPR16:
+        case KEYDB_SEARCH_MODE_FPR20:
+        case KEYDB_SEARCH_MODE_FPR:
+          found++;
+          break;
+        default:
+          break;
+        }
+    }
+  return found && found == ctx->nitems;
+}
+
+
 /* The main function to lookup a key.  On success the found keyblock
    is stored at RET_KEYBLOCK and also in CTX.  If WANT_SECRET is true
    a corresponding secret key is required.  */
@@ -2534,9 +2573,21 @@ lookup (getkey_ctx_t ctx, kbnode_t *ret_keyblock, int want_secret)
   int rc;
   int no_suitable_key = 0;
 
-  rc = 0;
-  while (!(rc = keydb_search (ctx->kr_handle, ctx->items, ctx->nitems, NULL)))
+  for (;;)
     {
+      rc = keydb_search (ctx->kr_handle, ctx->items, ctx->nitems, NULL);
+      /* Skip over all legacy keys but only if they are not requested
+         by fingerprints.
+         Fixme: The lower level keydb code should actually do that but
+         then it would be harder to report the number of skipped
+         legacy keys during import. */
+      if (gpg_err_code (rc) == GPG_ERR_LEGACY_KEY
+          && !(ctx->nitems && ctx->items->mode == KEYDB_SEARCH_MODE_FIRST)
+          && !search_modes_are_fingerprint (ctx))
+        continue;
+      if (rc)
+        break;
+
       /* If we are searching for the first key we have to make sure
          that the next iteration does not do an implicit reset.
          This can be triggered by an empty key ring. */
@@ -2570,6 +2621,11 @@ lookup (getkey_ctx_t ctx, kbnode_t *ret_keyblock, int want_secret)
       /* Release resources and continue search. */
       release_kbnode (ctx->keyblock);
       ctx->keyblock = NULL;
+      /* We need to disable the caching so that for an exact key search we
+         won't get the result back from the cache and thus end up in an
+         endless loop.  Disabling this here is sufficient because although
+         the result may have been cached, if won't be used then.  */
+      keydb_disable_caching (ctx->kr_handle);
     }
 
 found:
@@ -2582,7 +2638,8 @@ found:
       *ret_keyblock = ctx->keyblock; /* Return the keyblock.  */
       ctx->keyblock = NULL;
     }
-  else if (gpg_err_code (rc) == GPG_ERR_NOT_FOUND && no_suitable_key)
+  else if ((gpg_err_code (rc) == GPG_ERR_NOT_FOUND
+            || gpg_err_code (rc) == GPG_ERR_LEGACY_KEY) && no_suitable_key)
     rc = want_secret? GPG_ERR_UNUSABLE_SECKEY : GPG_ERR_UNUSABLE_PUBKEY;
   else if (gpg_err_code (rc) == GPG_ERR_NOT_FOUND)
     rc = want_secret? GPG_ERR_NO_SECKEY : GPG_ERR_NO_PUBKEY;
@@ -2719,34 +2776,69 @@ enum_secret_keys (void **context, PKT_public_key *sk)
 /* Return a string with a printable representation of the user_id.
  * this string must be freed by xfree.   */
 static char *
-get_user_id_string (u32 * keyid)
+get_user_id_string (u32 * keyid, int mode, size_t *r_len)
 {
   user_id_db_t r;
+  keyid_list_t a;
   int pass = 0;
+  char *p;
+
   /* Try it two times; second pass reads from key resources.  */
   do
     {
       for (r = user_id_db; r; r = r->next)
 	{
-	  keyid_list_t a;
 	  for (a = r->keyids; a; a = a->next)
 	    {
 	      if (a->keyid[0] == keyid[0] && a->keyid[1] == keyid[1])
 		{
-		  return xasprintf ("%s %.*s", keystr (keyid), r->len, r->name);
+                  if (mode == 2)
+                    {
+                      /* An empty string as user id is possible.  Make
+                         sure that the malloc allocates one byte and
+                         does not bail out.  */
+                      p = xmalloc (r->len? r->len : 1);
+                      memcpy (p, r->name, r->len);
+                      if (r_len)
+                        *r_len = r->len;
+                    }
+                  else
+                    {
+                      if (mode)
+                        p = xasprintf ("%08lX%08lX %.*s",
+                                       (ulong) keyid[0], (ulong) keyid[1],
+                                       r->len, r->name);
+                      else
+                        p = xasprintf ("%s %.*s", keystr (keyid),
+                                       r->len, r->name);
+                      if (r_len)
+                        *r_len = strlen (p);
+                    }
+
+                  return p;
 		}
 	    }
 	}
     }
   while (++pass < 2 && !get_pubkey (NULL, keyid));
-  return xasprintf ("%s [?]", keystr (keyid));
+
+  if (mode == 2)
+    p = xstrdup (user_id_not_found_utf8 ());
+  else if (mode)
+    p = xasprintf ("%08lX%08lX [?]", (ulong) keyid[0], (ulong) keyid[1]);
+  else
+    p = xasprintf ("%s [?]", keystr (keyid));
+
+  if (r_len)
+    *r_len = strlen (p);
+  return p;
 }
 
 
 char *
 get_user_id_string_native (u32 * keyid)
 {
-  char *p = get_user_id_string (keyid);
+  char *p = get_user_id_string (keyid, 0, NULL);
   char *p2 = utf8_to_native (p, strlen (p), 0);
   xfree (p);
   return p2;
@@ -2756,65 +2848,18 @@ get_user_id_string_native (u32 * keyid)
 char *
 get_long_user_id_string (u32 * keyid)
 {
-  user_id_db_t r;
-  keyid_list_t a;
-  int pass = 0;
-  /* Try it two times; second pass reads from key resources.  */
-  do
-    {
-      for (r = user_id_db; r; r = r->next)
-	{
-	  for (a = r->keyids; a; a = a->next)
-	    {
-	      if (a->keyid[0] == keyid[0] && a->keyid[1] == keyid[1])
-		{
-		  return xasprintf ("%08lX%08lX %.*s",
-                                    (ulong) keyid[0], (ulong) keyid[1],
-                                    r->len, r->name);
-		}
-	    }
-	}
-    }
-  while (++pass < 2 && !get_pubkey (NULL, keyid));
-  return xasprintf ("%08lX%08lX [?]", (ulong) keyid[0], (ulong) keyid[1]);
+  return get_user_id_string (keyid, 1, NULL);
 }
 
 
-/* Please try to use get_user_id_native instead of this one.  */
+/* Please try to use get_user_byfpr instead of this one.  */
 char *
 get_user_id (u32 * keyid, size_t * rn)
 {
-  user_id_db_t r;
-  char *p;
-  int pass = 0;
-
-  /* Try it two times; second pass reads from key resources.  */
-  do
-    {
-      for (r = user_id_db; r; r = r->next)
-	{
-	  keyid_list_t a;
-	  for (a = r->keyids; a; a = a->next)
-	    {
-	      if (a->keyid[0] == keyid[0] && a->keyid[1] == keyid[1])
-		{
-                  /* An empty string as user id is possible.  Make
-                     sure that the malloc allocates one byte and does
-                     not bail out.  */
-		  p = xmalloc (r->len? r->len : 1);
-		  memcpy (p, r->name, r->len);
-		  *rn = r->len;
-		  return p;
-		}
-	    }
-	}
-    }
-  while (++pass < 2 && !get_pubkey (NULL, keyid));
-  p = xstrdup (user_id_not_found_utf8 ());
-  *rn = strlen (p);
-  return p;
+  return get_user_id_string (keyid, 2, rn);
 }
 
+
 /* Please try to use get_user_id_byfpr_native instead of this one.  */
 char *
 get_user_id_native (u32 * keyid)
@@ -2828,7 +2873,7 @@ get_user_id_native (u32 * keyid)
 
 
 /* Return a user id from the caching by looking it up using the FPR
-   which mustbe of size MAX_FINGERPRINT_LEN.  */
+   which must be of size MAX_FINGERPRINT_LEN.  */
 char *
 get_user_id_byfpr (const byte *fpr, size_t *rn)
 {
@@ -2884,6 +2929,9 @@ get_ctx_handle (GETKEY_CTX ctx)
 static void
 free_akl (struct akl *akl)
 {
+  if (! akl)
+    return;
+
   if (akl->spec)
     free_keyserver_spec (akl->spec);
 
@@ -2936,10 +2984,8 @@ parse_auto_key_locate (char *options)
       else if (ascii_strcasecmp (tok, "cert") == 0)
 	akl->type = AKL_CERT;
 #endif
-#ifdef USE_DNS_PKA
       else if (ascii_strcasecmp (tok, "pka") == 0)
 	akl->type = AKL_PKA;
-#endif
       else if ((akl->spec = parse_keyserver_uri (tok, 1)))
 	akl->type = AKL_SPEC;
       else
@@ -3012,8 +3058,14 @@ have_secret_key_with_kid (u32 *keyid)
   desc.mode = KEYDB_SEARCH_MODE_LONG_KID;
   desc.u.kid[0] = keyid[0];
   desc.u.kid[1] = keyid[1];
-  while (!result && !(err = keydb_search (kdbhd, &desc, 1, NULL)))
+  while (!result)
     {
+      err = keydb_search (kdbhd, &desc, 1, NULL);
+      if (gpg_err_code (err) == GPG_ERR_LEGACY_KEY)
+        continue;
+      if (err)
+        break;
+
       err = keydb_get_keyblock (kdbhd, &keyblock);
       if (err)
         {
@@ -3039,6 +3091,7 @@ have_secret_key_with_kid (u32 *keyid)
 	}
       release_kbnode (keyblock);
     }
+
   keydb_release (kdbhd);
   return result;
 }
